/*-
 * Copyright (c) 2004 - 2011 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      CTPP2VMCodeStack.hpp
 *
 * $CTPP$
 */
#ifndef _CTPP2_VM_CODE_STACK_HPP__
#define _CTPP2_VM_CODE_STACK_HPP__ 1

/**
  @file CTPP2VMCodeStack.hpp
  @brief Virtual Machine stack
*/

#include "CDT.hpp"

namespace CTPP // C++ Template Engine
{
/**
  @class VMCodeStack CTPP2VMCodeStack.hpp <CTPP2VMCodeStack.hpp>
  @brief Virtual Machine stack

<pre>
    +----+  <-- iMaxStackSize
    |void|
    |void|
    |----|  <-- iStackPointer
    |addr|
    |addr|
    | .. |
    |addr|
    |addr|
    | .. |
    |addr|
    +----+  <-- 0
</pre>

*/
class CTPP2DECL VMCodeStack
{
public:
	/**
	  @brief Constructor
	  @param iIMaxStackSize - maximal stack size
	*/
	VMCodeStack(const INT_32  iIMaxStackSize = 4096);

	/**
	  @brief Push address into stack
	  @param iAddress - address to push
	  @return stack depth
	*/
	void PushAddress(const UINT_32  iAddress);

	/**
	  @brief Remove top stack element
	  @return stack depth
	*/
	UINT_32 PopAddress();

	/**
	  @brief Reset code stack to default state
	*/
	void Reset();

	/**
	  @brief A destructor
	*/
	~VMCodeStack() throw();
private:

	/**
	  @brief Copy constructor
	*/
	VMCodeStack(const VMCodeStack & oRhs);

	/**
	  @brief Copy operator
	*/
	VMCodeStack & operator=(const VMCodeStack & oRhs);

	/** Maximal stack size */
	const INT_32    iMaxStackSize;
	/** Current stack size */
	INT_32          iStackPointer;
	/** Stack array        */
	UINT_32       * aStack;
};

} // namespace CTPP
#endif // _CTPP2_VM_CODE_STACK_HPP__
// End.
